@@ -6,8 +6,6 @@ from openpyxl.styles import Font, Alignment
 import pandas as pd
 import pickle
 
-import gspread
-from oauth2client.service_account import ServiceAccountCredentials
 from df2gspread import df2gspread as d2g
 
 __author__ = 'sebastian'
@@ -98,13 +96,19 @@ def save_players_sheet(players, upload=False):
 def upload_sheet_from_df(spreadsheet_id, sheet_name, df, headers, upload_index=False):
     """ Saves headers and df data into given sheet_name.
         If sheet_name does not exist, it will be created. """
-    ws = d2g.upload(df, spreadsheet_id, sheet_name, row_names=upload_index, df_size=True)
+    print("<<<Saving\t", sheet_name, "\tin\t", spreadsheet_id)
+    try:
+        credentials = get_credentials()
+        ws = d2g.upload(df, spreadsheet_id, sheet_name, row_names=upload_index, df_size=True, credentials=credentials)
 
-    # Concatenation of header cells values to be updated in batch mode
-    cell_list = ws.range("A1:" + d2g.gspread.utils.rowcol_to_a1(row=1, col=len(headers)))
-    for i, value in enumerate(headers):
-        cell_list[i].value = value
-    ws.update_cells(cell_list)
+        # Concatenation of header cells values to be updated in batch mode
+        cell_list = ws.range("A1:" + d2g.gspread.utils.rowcol_to_a1(row=1, col=len(headers)))
+        for i, value in enumerate(headers):
+            cell_list[i].value = value
+        ws.update_cells(cell_list)
+
+    except FileNotFoundError:
+        print("<<<FAILED to upload\t", sheet_name, "\tin\t", spreadsheet_id)
 
 
 def load_players_sheet():
@@ -206,7 +210,7 @@ def _get_writer_mode(xlsx_file_path):
 def _get_writer(xlsx_filename, sheet_name):
     writer = pd.ExcelWriter(xlsx_filename, engine='openpyxl', mode=_get_writer_mode(xlsx_filename))
     if sheet_name in writer.book.sheetnames:
-        writer.book.remove_sheet(writer.book.get_sheet_by_name(sheet_name))
+        writer.book.remove(writer.book[sheet_name])
     print("<<<Saving\t", sheet_name, "\tin\t", xlsx_filename)
 
     return writer
@@ -266,7 +270,7 @@ def publish_rating_sheet(tournaments, rankings, players, tid, prev_tid, upload=F
         sorted_rankings_df.to_excel(writer, sheet_name=sheet_name, index=False, header=headers, columns=columns)
 
         # publish and format tournament metadata
-        ws = writer.book.get_sheet_by_name(sheet_name)
+        ws = writer.book[sheet_name]
         _publish_tournament_metadata(ws, tournaments[tid])
         _bold_and_center(ws, to_bold, to_center)
 
@@ -343,7 +347,7 @@ def publish_rating_details_sheet(tournaments, rankings, players, tid, prev_tid,
         rating_details.to_excel(writer, sheet_name=sheet_name, index=False, header=headers, columns=columns)
 
         # publish and format tournament metadata
-        ws = writer.book.get_sheet_by_name(sheet_name)
+        ws = writer.book[sheet_name]
         _publish_tournament_metadata(ws, tournaments[tid])
         _bold_and_center(ws, to_bold, to_center)
 
@@ -371,7 +375,7 @@ def publish_championship_details_sheet(tournaments, rankings, players, tid, prev
         championship_details.to_excel(writer, sheet_name=sheet_name, index=False, header=headers, columns=columns)
 
         # publish and format tournament metadata
-        ws = writer.book.get_sheet_by_name(sheet_name)
+        ws = writer.book[sheet_name]
         _publish_tournament_metadata(ws, tournaments[tid])
         _bold_and_center(ws, to_bold, to_center)
 
@@ -390,7 +394,7 @@ def publish_statistics_sheet(tournaments, rankings, players, tid, prev_tid, uplo
     with _get_writer(xlsx_filename, sheet_name) as writer:
         stats.to_excel(writer, sheet_name=sheet_name, index=True, header=headers, index_label=cfg["labels"]["tid"])
 
-        ws = writer.book.get_sheet_by_name(sheet_name)
+        ws = writer.book[sheet_name]
         ws.insert_rows(0, 1)
         ws["B1"] = "Acumulado"  # FIXME this should be read from cfg
         ws.merge_cells('B1:H1')
@@ -453,7 +457,7 @@ def publish_championship_sheets(tournaments, rankings, players, tid, prev_tid, u
             sorted_ranking.to_excel(writer, sheet_name=sheet_name, index=False, header=headers, columns=_columns)
 
             # publish and format tournament metadata
-            ws = writer.book.get_sheet_by_name(sheet_name)
+            ws = writer.book[sheet_name]
             _publish_tournament_metadata(ws, tournaments[tid])
             _bold_and_center(ws, to_bold, to_center)  # FIXME This should be part of publish metadata, to merge the right cells
 
@@ -461,26 +465,47 @@ def publish_championship_sheets(tournaments, rankings, players, tid, prev_tid, u
             load_and_upload_sheet(xlsx_filename, sheet_name, cfg["io"]["temporal_spreadsheet_id"])
 
 
+def in_colab():
+    # Verify if it is running on colab
+    try:
+        import google.colab
+        _in_colab = True
+    except:
+        _in_colab = False
+
+    return _in_colab
+
+
+def get_credentials():
+    if in_colab():
+        from google.colab import auth
+        auth.authenticate_user()
+        from oauth2client.client import GoogleCredentials
+        credentials = GoogleCredentials.get_application_default()
+    else:
+        credentials = d2g.get_credentials()
+
+    return credentials
+
+
 def _get_gc():
-    # Drive authorization
-    scope = ['https://spreadsheets.google.com/feeds',
-             'https://www.googleapis.com/auth/drive']
-    key_filename = models.user_config_path + "/key-for-gspread.json"
     gc = None
     try:
-        credentials = ServiceAccountCredentials.from_json_keyfile_name(key_filename, scope)
-        gc = gspread.authorize(credentials)
+        credentials = get_credentials()
+        gc = d2g.gspread.authorize(credentials)
     except FileNotFoundError:
         print("The .json key file has not been configured. Upload will fail.")
     except OSError:
         print("Connection failure. Upload will fail.")
+
     return gc
 
 
 def load_and_upload_sheet(filename, sheet_name, spreadsheet_id):
     print("<<<Saving\t", sheet_name, "\tin\t", spreadsheet_id)
+    credentials = get_credentials()
     df = pd.read_excel(filename, sheet_name, index_col=None, header=None, na_filter=False)
-    d2g.upload(df, spreadsheet_id, sheet_name, row_names=False, col_names=False, df_size=True)
+    d2g.upload(df, spreadsheet_id, sheet_name, row_names=False, col_names=False, df_size=True, credentials=credentials)
 
 
 def create_n_tour_sheet(spreadsheet_id, tid):
@@ -512,7 +537,7 @@ def create_n_tour_sheet(spreadsheet_id, tid):
             dup_ws.update_acell('A1', dup_cell_value.replace(first_key, replacement_key))
             print("<<<Creating\t", new_sheetname, "\tfrom\t", sheetname, "\tin\t", spreadsheet_id)
         else:
-            print("FAILED TO DUPLICATE\t", first_key, "\t not exist in\t", spreadsheet_id)
+            print("FAILED TO DUPLICATE\t", first_key, "\t do not exist in\t", spreadsheet_id)
 
 
 def publish_to_web(tid, show_on_web=False):
@@ -524,8 +549,8 @@ def publish_to_web(tid, show_on_web=False):
 def load_temp_players_ranking():
     """returns players_temp, ranking_temp"""
     # Loading temp ranking and players. It shuould be deleted after a successful preprocessing
-    players_temp_file = cfg["io"]["players_temp_file"]
-    ranking_temp_file = cfg["io"]["ranking_temp_file"]
+    players_temp_file = os.path.join(cfg["io"]["data_folder"], cfg["io"]["players_temp_file"])
+    ranking_temp_file = os.path.join(cfg["io"]["data_folder"], cfg["io"]["ranking_temp_file"])
 
     if os.path.exists(players_temp_file):
         with open(players_temp_file, 'rb') as f:
@@ -547,9 +572,9 @@ def load_temp_players_ranking():
 def save_temp_players_ranking(players_temp, ranking_temp):
     """returns players_temp, ranking_temp"""
     # Loading temp ranking and players. It shuould be deleted after a successful preprocessing
-    players_temp_file = cfg["io"]["players_temp_file"]
-    ranking_temp_file = cfg["io"]["ranking_temp_file"]
-    print("<Saving\t\tTemps to resume preprocessing (if necessary)", ranking_temp_file, players_temp_file)
+    players_temp_file = os.path.join(cfg["io"]["data_folder"], cfg["io"]["players_temp_file"])
+    ranking_temp_file = os.path.join(cfg["io"]["data_folder"], cfg["io"]["ranking_temp_file"])
+    print("<Saving\t\tTemps to resume preprocessing (if necessary)", ranking_temp_file, players_temp_file, "\n")
     with open(players_temp_file, 'wb') as ptf, open(ranking_temp_file, 'wb') as rtf:
         # Pickle the 'data' dictionary using the highest protocol available.
         pickle.dump(players_temp, ptf, pickle.HIGHEST_PROTOCOL)
@@ -557,8 +582,8 @@ def save_temp_players_ranking(players_temp, ranking_temp):
 
 
 def remove_temp_players_ranking():
-    players_temp_file = cfg["io"]["players_temp_file"]
-    ranking_temp_file = cfg["io"]["ranking_temp_file"]
+    players_temp_file = os.path.join(cfg["io"]["data_folder"], cfg["io"]["players_temp_file"])
+    ranking_temp_file = os.path.join(cfg["io"]["data_folder"], cfg["io"]["ranking_temp_file"])
     print("Removing temp files created to resume preprocessing", players_temp_file, ranking_temp_file)
     if os.path.exists(players_temp_file):
         os.remove(players_temp_file)
@@ -573,13 +598,13 @@ def save_to_pickle(players=None, rankings=None, tournaments=None):
 
     for obj, fn in objects_filenames:
         print(f'<<<Saving\t{fn}\tin\t{cfg["io"]["data_folder"]}')
-        with open(os.path.join(cfg["io"]["data_folder"] + fn), 'wb') as fo:
+        with open(os.path.join(cfg["io"]["data_folder"], fn), 'wb') as fo:
             pickle.dump(obj, fo, pickle.HIGHEST_PROTOCOL)
 
 
 def load_from_pickle(filename):
     print(f'>>>Loading\t{filename}\tfrom\t{cfg["io"]["data_folder"]}')
-    with open(os.path.join(cfg["io"]["data_folder"] + filename), 'rb') as fo:
+    with open(os.path.join(cfg["io"]["data_folder"], filename), 'rb') as fo:
         obj = pickle.load(fo)
 
     return obj
